/**
 * Portalgun server-sent plugin made by Lorgan
 *
 * Portalgun sprite made by "Novally" http://novally.deviantart.com/art/Portal-Gun-Sprites-265796048
 */

globalvar PORTAL;
PORTAL = id;

// Constants
PORTAL.PLACE_PORTAL = 1;
PORTAL.DESTROY_PORTAL = 2;
PORTAL.PLACE_PEDESTAL = 3;
PORTAL.EQUIP = 4;
PORTAL.UNEQUIP = 5;
PORTAL.FULL_STATE = 6;
PORTAL.PICK_UP = 7;
PORTAL.DROP = 8;
PORTAL.SYNC_CUBE = 9;
PORTAL.BUTTON_PRESS = 10;
PORTAL.RIGHT_CLICK = 11;
PORTAL.SYNC_TURRET = 12;

PORTAL.INTERACT = 1;
PORTAL.ALT = 2;
PORTAL.RIGHT = 3;

// Networking
PORTAL.sendBuffer = buffer_create();
PORTAL.portals = ds_list_create();
PORTAL.pedestals = ds_list_create();
PORTAL.cubes = ds_list_create();
PORTAL.buttons = ds_list_create();

// Sprites
PORTAL.PortalgunS = sprite_add(directory + "/Sprites/PortalgunS.png", 1, true, false, 7, 10);
PORTAL.PortalgunParticlesS = sprite_add(directory + "/Sprites/PortalgunParticlesS.png", 1, true, false, 7, 10);
PORTAL.PortalS = sprite_add(directory + "/Sprites/PortalS.png", 2, true, false, 32, 32);
PORTAL.ProjectileS = sprite_add(directory + "/Sprites/PortalProjectileS.png", 1, true, false, 4, 4);
PORTAL.PropsS = sprite_add(directory + "/Sprites/PortalPropsS.png", 13, true, false, 30, 30);
PORTAL.PortalCrosshairS = sprite_add(directory + "/Sprites/PortalCrosshairS.png", 2, true, false, 15, 15);
PORTAL.PortalKL = sprite_add(directory + "/Sprites/PortalKL.png", 2, true, false, 14, 9);
PORTAL.CubeS = sprite_add(directory + "/Sprites/CubesS.png", 4, true, false, 12, 12);
PORTAL.TurretS = sprite_add(directory + "/Sprites/TurretS.png", 1, true, false, 14, 25);
PORTAL.TurretGunS = sprite_add(directory + "/Sprites/TurretGunS.png", 1, true, false, 12, 14);
PORTAL.DecalsS = sprite_add(directory + "/Sprites/DecalsS.png", 4, false, false, 8, 8);
PORTAL.DoorS = sprite_add(directory + "/Sprites/DoorS.png", 2, false, false, 30, 30);
PORTAL.RocketTurretBaseS = sprite_add(directory + "/Sprites/RocketTurretBaseS.png", 12, false, false, 22, 27);
PORTAL.RocketTurretS = sprite_add(directory + "/Sprites/RocketTurretS.png", 2, false, false, 15, 12);
PORTAL.RocketTurretRotateS = sprite_add(directory + "/Sprites/RocketTurretRotateS.png", 5, false, false, 18, 10);
PORTAL.ExcursionFunnelS = sprite_add(directory + "/Sprites/ExcursionFunnelS.png", 14, false, false, 0, 16);

// Fix collision mask
sprite_collision_mask(PORTAL.PortalS, false, 2, 26, 15, 38, 48, 1, 0);
sprite_collision_mask(PORTAL.PropsS, true, 0, -1, -1, -1, -1, 0, 0);
sprite_collision_mask(PORTAL.TurretS, false, 2, 13, 0, 20, 31, 1, 0);
sprite_collision_mask(PORTAL.CubeS, false, 2, 2, 2, 22, 22, 1, 0);

// Sound
PORTAL.ShotSnd = sound_add(directory + "/Sounds/PortalShot.wav", 0, true);
PORTAL.FizzleSnd = sound_add(directory + "/Sounds/Fizzle.wav", 0, true);
PORTAL.ButtonSnd = sound_add(directory + "/Sounds/Button.wav", 0, true);

// Controls
ini_open("gg2.ini");
PORTAL.interactKey = ini_read_real("Plugins", "portal_interact", ord("Q"));
PORTAL.altKey = ini_read_real("Plugins", "portal_alt", ord("R"));
ini_close();
global.specialBkp = global.special;

// Kill icon
createGlobalDamageSource("DAMAGE_SOURCE_PORTAL", PORTAL.PortalKL);

// Prototype: A game element that receives a signal.
PORTAL.Receiver = object_add();
object_set_sprite(PORTAL.Receiver, PORTAL.PropsS);
object_set_parent(PORTAL.Receiver, BulletWall);

object_event_add(PORTAL.Receiver, ev_create, 0, '
    open = false;
    receiveId = 0;
    active = false;
    invert = false;
    buttons = -1;

    image_speed = 0;
');

object_event_add(PORTAL.Receiver, ev_step, ev_step_begin, '');
object_event_add(PORTAL.Receiver, ev_step, ev_step_normal, '
    if (buttons == -1) {
        buttons = ds_list_create();
        var i;
        for (i = 0; i < ds_list_size(PORTAL.buttons); i += 1) {
            button = ds_list_find_value(PORTAL.buttons, i);
            if (button.receiveId == receiveId) {
                ds_list_add(buttons, button);
            }
        }
    }

    if (ds_list_size(buttons) == 0) {
        active = invert;
    } else {
        active = !invert;
    }

    var i;
    for (i = 0; i < ds_list_size(buttons); i += 1) {
        button = ds_list_find_value(buttons, i);
        if (button.pressed == false) {
            active = invert;
            break;
        }
    }
');

object_event_add(PORTAL.Receiver, ev_destroy, 0, '
    if (buttons != -1) {
        ds_list_destroy(buttons);
    }
');

object_event_add(PORTAL.Receiver, ev_other, ev_user1, '
    receiveId = readProperty(other.properties, "receiveId", REAL, receiveId);
    invert = readProperty(other.properties, "invert", BOOL, invert);
    image_angle = readProperty(other.properties, "angle", REAL, image_angle);
');

// Prototype: A game element that sends a signal.
PORTAL.Sender = object_add();
object_set_sprite(PORTAL.Sender, PORTAL.PropsS);
object_set_parent(PORTAL.Sender, DropdownPlatform);

object_event_add(PORTAL.Sender, ev_create, 0, '
    receiveId = 0;
    decalId = 0;
    pressed = false;

    image_speed = 0;
    ds_list_add(PORTAL.buttons, id);
');

object_event_add(PORTAL.Sender, ev_alarm, 0, '
    pressed = false;
');

object_event_add(PORTAL.Sender, ev_step, ev_step_begin, '');

object_event_add(PORTAL.Sender, ev_destroy, 0, '
    ds_list_delete(PORTAL.buttons, ds_list_find_index(PORTAL.buttons, id));
');

object_event_add(PORTAL.Sender, ev_other, ev_user1, '
    receiveId = readProperty(other.properties, "receiveId", REAL, receiveId);
    decalId = readProperty(other.properties, "decalId", REAL, decalId);
    image_angle = readProperty(other.properties, "angle", REAL, image_angle);
');

// scripts
execute_file(directory + "/scripts.gml");
execute_file(directory + "/menu.gml");
execute_file(directory + "/portal.gml");
execute_file(directory + "/turrets.gml");
execute_file(directory + "/projectiles.gml");
execute_file(directory + "/entities.gml");
execute_file(directory + "/particles.gml");
execute_file(directory + "/chat_ext.gml");

execute_file(directory + "/lorganbase.gml");
ds_list_add(lorganbase_plugins, "portal");

// More constants after the extra scripts are loaded.
PORTAL.BEAMCOLLIDERS = ds_list_create();
ds_list_add(PORTAL.BEAMCOLLIDERS, Obstacle);
ds_list_add(PORTAL.BEAMCOLLIDERS, Generator);
ds_list_add(PORTAL.BEAMCOLLIDERS, Character);
ds_list_add(PORTAL.BEAMCOLLIDERS, Sentry);
ds_list_add(PORTAL.BEAMCOLLIDERS, TeamGate);
ds_list_add(PORTAL.BEAMCOLLIDERS, IntelGate);
ds_list_add(PORTAL.BEAMCOLLIDERS, ControlPointSetupGate);
ds_list_add(PORTAL.BEAMCOLLIDERS, BulletWall);
ds_list_add(PORTAL.BEAMCOLLIDERS, DropdownPlatform);
ds_list_add(PORTAL.BEAMCOLLIDERS, PORTAL.Portal);

PORTAL.ROCKETCOLLIDERS = ds_list_create();
ds_list_add(PORTAL.ROCKETCOLLIDERS, Obstacle);
ds_list_add(PORTAL.ROCKETCOLLIDERS, TeamGate);
ds_list_add(PORTAL.ROCKETCOLLIDERS, IntelGate);
ds_list_add(PORTAL.ROCKETCOLLIDERS, ControlPointSetupGate);
ds_list_add(PORTAL.ROCKETCOLLIDERS, BulletWall);
ds_list_add(PORTAL.ROCKETCOLLIDERS, PORTAL.Portal);

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Handle networking and input
object_event_add(PlayerControl, ev_step, ev_step_begin, '
    var receiveBuffer, obj;
    obj = -1;
    while(PluginPacketGetBuffer(PORTAL.packetID) != -1) {
        receiveBuffer = PluginPacketGetBuffer(PORTAL.packetID);
        if (global.isHost == false) {
            var action;
            action = read_ubyte(receiveBuffer);
            switch(action) {
                case PORTAL.PLACE_PORTAL:
                    execute_string(PORTAL.PlacePortal, read_ushort(receiveBuffer) / 5, read_ushort(receiveBuffer) / 5, read_ubyte(receiveBuffer) * 2, make_color_rgb(read_ubyte(receiveBuffer), read_ubyte(receiveBuffer), read_ubyte(receiveBuffer)));
                    break;
                case PORTAL.DESTROY_PORTAL:
                    execute_string(PORTAL.DestroyPortal, read_ubyte(receiveBuffer));
                    break;
                case PORTAL.PLACE_PEDESTAL:
                    execute_string(PORTAL.PlacePedestal, read_ushort(receiveBuffer) / 5, read_ushort(receiveBuffer) / 5, make_color_rgb(read_ubyte(receiveBuffer), read_ubyte(receiveBuffer), read_ubyte(receiveBuffer)), read_ushort(receiveBuffer));
                    break;
                case PORTAL.EQUIP:
                    execute_string(PORTAL.Equip, read_ubyte(receiveBuffer), read_ubyte(receiveBuffer), make_color_rgb(read_ubyte(receiveBuffer), read_ubyte(receiveBuffer), read_ubyte(receiveBuffer)));
                    break;
                case PORTAL.UNEQUIP:
                    execute_string(PORTAL.Unequip, read_ubyte(receiveBuffer), read_ubyte(receiveBuffer));
                    break;
                case PORTAL.FULL_STATE:
                    execute_string(PORTAL.ReadFullState, receiveBuffer);
                    break;
                case PORTAL.PICK_UP:
                    execute_string(PORTAL.Pickup, read_ubyte(receiveBuffer), read_ubyte(receiveBuffer));
                    break;
                case PORTAL.DROP:
                    execute_string(PORTAL.Drop, read_ubyte(receiveBuffer), read_ubyte(receiveBuffer));
                    break;
                case PORTAL.SYNC_CUBE:
                    obj = PORTAL.Cube;
                case PORTAL.SYNC_TURRET:
                    if (obj != noone) {
                        obj = PORTAL.Turret;
                    }

                    var cubeId, alpha;
                    cubeId = read_ubyte(receiveBuffer);
                    alpha = read_ubyte(receiveBuffer) / 255;
                    if (alpha == 0) {
                        execute_string(PORTAL.SyncCube, cubeId, 0, -999, -999, obj);
                    } else {
                        execute_string(PORTAL.SyncCube, cubeId, alpha, read_ushort(receiveBuffer) / 5, read_ushort(receiveBuffer) / 5, obj);
                    }
                    break;
                case PORTAL.BUTTON_PRESS:
                    execute_string(PORTAL.ButtonPress, read_ubyte(receiveBuffer));
                    break;
                case PORTAL.RIGHT_CLICK:
                    execute_string(PORTAL.RightClick, read_ubyte(receiveBuffer));
                    break;
                break;
            }
        } else {
            var player, foundPedestal, key;
            player = PluginPacketGetPlayer(PORTAL.packetID);
            key = read_ubyte(receiveBuffer);
            if (player == noone) { // WHY IS THIS NOONE THIS IS STUPID
                player = global.myself;
            }

            if (player != -1) {
                if (player.object != -1) {
                    if (key == PORTAL.INTERACT) {
                        foundPedestal = false;

                        with(PORTAL.Pedestal) {
                            if (distance_to_point(player.object.x, player.object.y) < 20) {
                                if (hasPortalgun == true) {
                                    execute_string(PORTAL.SendEquip, player, id, color);
                                    foundPedestal = true;
                                } else if (player.object.currentWeapon.object_index == PORTAL.Portalgun) {
                                    if (player.object.currentWeapon.holding == noone) {
                                        execute_string(PORTAL.SendUnequip, player, id);
                                        foundPedestal = true;
                                    }
                                }
                            }

                            if (foundPedestal == true) {
                                break;
                            }
                        }

                        if (foundPedestal == false && player.object.currentWeapon.object_index == PORTAL.Portalgun) {
                            if (player.object.currentWeapon.holding != noone) {
                                var _x, _y, obstructed;
                                obstructed = false;
                                _x = player.object.currentWeapon.x + player.object.currentWeapon.xoffset * player.object.currentWeapon.image_xscale + lengthdir_x(45, player.object.aimDirection);
                                _y = player.object.currentWeapon.y + player.object.currentWeapon.yoffset + lengthdir_y(45, player.object.aimDirection) + player.object.equipmentOffset;
                                with(player.object.currentWeapon.holding) {
                                    if (place_meeting(_x, _y, CharacterCollider) == true || place_meeting(_x, _y, DropdownPlatform) == true) {
                                        obstructed = true;
                                    }
                                }

                                if (!obstructed) {
                                    execute_string(PORTAL.SendDrop, player, player.object.currentWeapon.holding);
                                }
                            } else {
                                var _x, _y, closest, dist, d, i, cube;
                                closest = noone;
                                dist = 40;
                                _x = player.object.currentWeapon.x + player.object.currentWeapon.xoffset * player.object.currentWeapon.image_xscale + lengthdir_x(45, player.object.aimDirection);
                                _y = player.object.currentWeapon.y + player.object.currentWeapon.yoffset + lengthdir_y(45, player.object.aimDirection) + player.object.equipmentOffset;
                                for (i = 0; i < ds_list_size(PORTAL.cubes); i += 1) {
                                    cube = ds_list_find_value(PORTAL.cubes, i);
                                    d = point_distance(_x, _y, cube.x, cube.y);
                                    if (d < dist) {
                                        closest = cube;
                                        dist = d;
                                    }
                                }

                                if (closest != noone) {
                                    execute_string(PORTAL.SendPickup, player, closest);
                                } else {
                                    with(PORTAL.Button) {
                                        if (distance_to_point(player.object.x, player.object.y) < 20) {
                                            execute_string(PORTAL.SendButtonPress, id);
                                        }
                                    }
                                }
                            }
                        } else if (foundPedestal == false) {
                            // You dont need the portalgun to press buttons.
                            with(PORTAL.Button) {
                                if (distance_to_point(player.object.x, player.object.y) < 20) {
                                    execute_string(PORTAL.SendButtonPress, id);
                                }
                            }
                        }
                    } else if (key == PORTAL.ALT) {
                        if (player.object.currentWeapon.object_index == PORTAL.Portalgun) {
                            with(player.object.currentWeapon.primary) {
                                execute_string(PORTAL.SendDestroyPortal, id);
                            }
                            with(player.object.currentWeapon.secondary) {
                                execute_string(PORTAL.SendDestroyPortal, id);
                            }
                        }
                    } else if (key == PORTAL.RIGHT) {
                        execute_string(PORTAL.SendRightClick, player);
                    }
                }
            }
        }

        PluginPacketPop(PORTAL.packetID);
    }

    var kickOpen, key;
    kickOpen = false;
    key = -1;

    if (instance_exists(ScoreTableController))
        if (ScoreTableController.showadmin)
            kickOpen = true;

    if (instance_exists(MenuController) || instance_exists(TeamSelectController) || instance_exists(ClassSelectController) || kickOpen == true || global.myself.object == -1) {
        exit;
    }

    // When were holding the portalgun we disable right click and send our own event instead.
    // This way we can bind the "special" action to the alt key so engineer, demo, sniper and heavy can use the portalgun correctly.
    if (global.myself.object.currentWeapon.object_index == PORTAL.Portalgun) {
        global.special = -1;

        if(mouse_check_button_pressed(mb_left) and global.specialBkp == MOUSE_LEFT) {
            key = PORTAL.RIGHT;
        } else if(mouse_check_button_pressed(mb_right) and global.specialBkp == MOUSE_RIGHT) {
            key = PORTAL.RIGHT;
        } else if(keyboard_check_pressed(global.specialBkp)) {
            key = PORTAL.RIGHT;
        }
    } else {
        global.special = global.specialBkp;
    }

    if (key == -1) {
        if (keyboard_check_pressed(PORTAL.interactKey) == true) {
            key = PORTAL.INTERACT;
        } else if (keyboard_check_pressed(PORTAL.altKey) == true && global.myself.object.currentWeapon.object_index == PORTAL.Portalgun) {
            if (instance_exists(global.myself.object.currentWeapon.primary) == true || instance_exists(global.myself.object.currentWeapon.secondary) == true) {
                key = PORTAL.ALT;
            } else {
                inputSpecial();
            }
        }
    }

    if (key != -1) {
        write_ubyte(PORTAL.sendBuffer, key);
        if (global.isHost == true) {
            PluginPacketSendTo(PORTAL.packetID, PORTAL.sendBuffer, global.myself);
        } else {
            PluginPacketSend(PORTAL.packetID, PORTAL.sendBuffer);
        }
    }
    buffer_clear(PORTAL.sendBuffer);
');

// Allow rebinding the special key when holding the portalgun.
object_event_add(AdvControlsController, ev_create, 0, '
    global.special = global.specialBkp;
');

object_event_add(AdvControlsController, ev_destroy, 0, '
    global.specialBkp = global.special;
');

// Send initial state to the client.
object_event_add(Player, ev_create, 0, '
    if (global.isHost) {
        alarm[1] = 1;
    }
');
object_event_add(Player, ev_alarm, 1, '
    execute_string(PORTAL.SendFullState);
');

// Character
object_event_add(Character, ev_create, 0, '
    lastPortal = noone;
    sourcePedestal = noone;
');

object_event_add(Character, ev_step, ev_step_begin, '
    oldHspeed = hspeed;
');

object_event_add(Character, ev_step, ev_step_normal, '
    if (moveStatus == 3) {
        // counteract the fast slowing down when coming out of a portal.
        if(abs(hspeed) > basemaxspeed * 2 or ((keyState|pressedKeys) & $60 and abs(hspeed) < basemaxspeed)) {
            hspeed *= delta_mult_skip(baseFriction);

        } else {
            hspeed *= delta_mult_skip(frictionFactor);
        }
    }

    if (place_meeting(x, y, PORTAL.Portal) == false) {
        lastPortal = noone;
    }
');

object_event_add(Character, ev_destroy, 0, '
    with(sourcePedestal) {
        hasPortalgun = true;
        alarm[0] = -1;
    }
');

object_event_add(Character, ev_collision, PORTAL.Portal, '
    // Dont make it /too/ easy.
    if (intel == true) {
        exit;
    }

    // Exit the portal before being teleported again.
    if (other.id == lastPortal) {
        exit;
    }

    // Teleport if close enough.
    if (instance_exists(other.pair) == true) {
        // Shoot out of the other portal with your current velocity.
        var tmpSpeed;
        tmpSpeed = speed;
        hspeed = lengthdir_x(tmpSpeed, other.pair.image_angle);
        vspeed = lengthdir_y(tmpSpeed, other.pair.image_angle);

        // Teleport to the pair and solve collisions.
        x = other.pair.x;
        y = other.pair.y;

        charSetSolids();
        move_outside_solid(other.pair.image_angle, 32);
        move_contact_solid(other.pair.image_angle + 180, 32);
        charUnsetSolids();

        lastPortal = other.pair;
        moveStatus = 3;
    }
');

object_event_add(Character, ev_collision, PORTAL.Plate, '
    if (other.pressed == false) {
        playsound(x, y, PORTAL.ButtonSnd);
    }

    other.pressed = true;
    other.alarm[0] = 2;
');

object_event_add(Character, ev_collision, PORTAL.EmancipationGrill, '
    if (currentWeapon.object_index == PORTAL.Portalgun) {
        with(currentWeapon) {
            event_user(14);
        }
    }
');

object_event_add(Character, ev_collision, PORTAL.Elevator, '
    if (global.isHost == true) {
        global.nextMap = nextMapInRotation();
        GameServer.impendingMapChange = 0; // Immediately go to the next map.
    }
');

object_event_add(Character, ev_collision, PORTAL.Notification, '
    if (global.isHost == true && other.notified == false) {
        other.notified = true;
        ServerMessageString(other.message, global.sendBuffer);

        var notice;
        with NoticeO instance_destroy();
        notice = instance_create(0, 0, NoticeO);
        notice.notice = NOTICE_CUSTOM;
        notice.message = other.message;
    }
');

object_event_add(Character, ev_other, ev_user12, '
    if(global.updateType == FULL_UPDATE) {
        var index;
        index = 254;
        if (currentWeapon.object_index == PORTAL.Portalgun) {
            if (currentWeapon.holding != noone) {
                index = ds_list_find_index(PORTAL.cubes, currentWeapon.holding);
            } else {
                index = 255;
            }
            write_ubyte(global.serializeBuffer, index);
            write_ubyte(global.serializeBuffer, currentWeapon.colorR);
            write_ubyte(global.serializeBuffer, currentWeapon.colorG);
            write_ubyte(global.serializeBuffer, currentWeapon.colorB);
            write_ubyte(global.serializeBuffer, ds_list_find_index(PORTAL.pedestals, sourcePedestal));
        } else {
            write_ubyte(global.serializeBuffer, index);
        }
    }
');

object_event_add(Character, ev_other, ev_user13, '
    if(global.updateType == FULL_UPDATE) {
        receiveCompleteMessage(global.serverSocket, 1, global.deserializeBuffer);
        var index;
        index = read_ubyte(global.deserializeBuffer)
        if (index != 254) {
            with(player.object.currentWeapon) {
                instance_destroy();
            }

            global.paramOwner = player.object;
            player.object.currentWeapon = instance_create(x, y, PORTAL.Portalgun);
            global.paramOwner = noone;

            receiveCompleteMessage(global.serverSocket, 4, global.deserializeBuffer);
            player.object.currentWeapon.colorR = read_ubyte(global.deserializeBuffer);
            player.object.currentWeapon.colorG = read_ubyte(global.deserializeBuffer);
            player.object.currentWeapon.colorB = read_ubyte(global.deserializeBuffer);
            player.object.sourcePedestal = read_ubyte(global.deserializeBuffer); // Well fix this up when we receive the full state.

            if (index != 255) {
                player.object.currentWeapon.holding = ds_list_find_value(PORTAL.cubes, index);
            }
        }
    }
');

object_event_add(Sentry, ev_collision, PORTAL.Portal, '
    execute_string(PORTAL.TeleportProjectile, id, other.id, true);

    // Make sure to sync the new location.
    oldVspeed = 999;
    lastPortal = other.id;
');

object_event_add(Sentry, ev_draw, 0, '
    if(vspeed == 0 && built == 1) {
        image_index = 11;
    } else if (currentWeapon != -1) {
        // Dont shoot while parashuting.
        currentWeapon.readyToShoot = false;
        currentWeapon.alarm[0] = currentWeapon.refireTime / global.delta_factor;
    }
');

object_event_add(Sentry, ev_other, ev_user2, '
    if (vspeed == 0) {
        hspeed = 0;
    }
');

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// The ultimate cheat: a player that does not show up on the scoreboard that's the owner of all damage inflicted by the map.
// I found one problem with this the amount of players sent to the lobby is now incorrect.
PORTAL.MapPlayer = object_add();
object_set_parent(PORTAL.MapPlayer, Player);
object_set_persistent(PORTAL.MapPlayer, true);
object_event_add(PORTAL.MapPlayer, ev_alarm, 5, '');

// Server sent plugins are executed at an akward time so I have to wait 1 frame before spawning glados.
alarm[0] = 1;
object_event_add(object_index, ev_alarm, 0, '
    if (global.isHost == true) {
        PORTAL.MapPlayerInstance = instance_create(0, 0, PORTAL.MapPlayer);
        PORTAL.MapPlayerInstance.name = "GLaDOS";
        PORTAL.MapPlayerInstance.team = 255; // Hide from scoreboard.
        PORTAL.MapPlayerInstance.socket = ds_list_find_value(global.players, 0).socket; // We get "kicked" if we dont have a valid socket, using the hosts socket should be ok.

        ds_list_add(global.players, PORTAL.MapPlayerInstance);
    } else {
        // Dont add to global.players! This player merely exists to prevent clientside errors.
        PORTAL.MapPlayerInstance = instance_create(0, 0 , PORTAL.MapPlayer);
    }
');
